/*****************************************************************************
    dump.c

    Diese Datei ist Teil des Programmpakets 'ACL-tftp'.
    Alle Rechte bei Andreas G. Schindler
                    Klappacher Str. 2i
                    64285 Darmstadt, Deutschland.

    Dieses Programm ist 'Freie Software'; Sie knnen es entsprechend den 
    Bedingungen der 'Allgemeinen ffentlichen GNU-Lizenz' (verffentlicht 
    durch die 'Free Software Foundation') weiter verbreiten und/oder 
    modifizieren. Massgeblich ist die Version 2 oder eine neuere Version
    dieser Lizenz. 

    Dieses Programm wird in der Hoffnung dass es ntzlich is weitergegeben,
    jedoch OHNE JEDE GARANTIE. Dieser Garantieauschluss bezieht sich auch
    auf Eigenschaften wie Marktfhigkeit oder Eignung zu einem bestimmten
    Zweck. Details finden sich in der 'Allgemeinen ffentlichen GNU-Lizenz'.

    Eine Kopie dieser Lizenz sollte in  Programmpaket, dem diese Datei
    entstammt, enthalten sein. Falls nicht, schreiben Sie an die 
    'Free Software Foundation, Inc.', 59 Temple Place, Suite 330, Boston, 
    MA  02111-1307  USA.


    This file is part of the 'ACL-tftp' Package
    Copyright (C) 1999-2005 by Andreas G. Schindler
                               Klappacher Str. 2i
                               Darmstadt, Germany.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************/
#include "auto.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#include "params.h"
#ifdef FILE_CACHE
#define CPROPLIST
#include "cache.h"
#endif /* FILE_CACHE */
#include "globals.h"
#include "netio.h"
#define  PERMTABLES
#include "acl.h"
#include "dll.h"
#include "dllapps.h"
#include "util.h"
#include "logfun.h"
#include "access.h"
#include "dump.h"

/*
        convert a direction type bitmask into a prefix character
*/

static char *dtype2prefix (int dtype)
{
    switch (dtype) {
        case RT_RRQ:
            return "<";
        case RT_WRQ:
            return ">";
    }
    return "";
}


static void aclDump (FILE *dump, acl_t *acl, int *anyflag, int increment)
{
    if (dump == NULL)
        return;
    if (!(*anyflag) && (acl->mask == IPMASK_ANY)) {
        fprintf (dump, _("any\n"));
        *anyflag += increment;
    }
    else if (acl->mask == IPMASK_HOST) {
        fprintf (dump, "%s %s\n", "host", xinet_ntoa(acl->addr, doResolv));
    }
    else {
        fprintf (dump, "%s ",  xinet_ntoa((acl->addr & acl->mask),0));
        fprintf (dump, "%s\n", xmask_ntoa(acl->mask));
    }
}


void timerDump(FILE *dump)
{
    dlt_t *dt;
    int gotAny=0;

    if (dump == NULL)
        return;
/*
	scan the timer list
*/
    if (timerListH != NULL)
        for (dt = NULL; (dt = getNextTimer(timerListH, dt)) != NULL;) 
            gotAny++;

    fprintf (dump, _("\nTotal number of timer lists allocated: %d\n"), 
             gotAny);

    if (!gotAny)
        return;

    for (dt = NULL; (dt = getNextTimer(timerListH, dt)) != NULL;) {
        fprintf (dump, _("\nTimer list %d: %s\n"),
                 dt->mark, timer2asc(&dt->times, T2A_XFER|T2A_SETUP|T2A_RET));
    }
}


void filterDump(FILE *dump)
{
    dlf_t *df;
    int gotAny=0;

    if (dump == NULL)
        return;
/*
	scan the filter list
*/

    fprintf (dump, _("\nFilter declarations\n"));
    if (filterListH != NULL) {
        for (df = NULL; (df = getNextFilter(filterListH, df)) != NULL;) {
            fprintf (dump, "%5d  %-30s  %-30s\n",
                     df->mark, 
                     (df->rxfilter == NULL) ? "" : df->rxfilter, 
                     (df->txfilter == NULL) ? "" : df->txfilter);
            gotAny++;
        }
    }
    if (!gotAny)
        fprintf (dump,_("        --none--\n"));
}

/*
	dump the limits list of thread restrictions
*/

void limitsDump(FILE *dump)
{
    dla_t *da;
    int gotAny=0;

    if (dump == NULL)
        return;

    fprintf (dump, _("\nLimits (thread restrictions)\n"));
    if (threadListH != NULL) {
        for (da = NULL; (da = getNextACL(threadListH, da)) != NULL;) {
            gotAny++;
            fprintf (dump, "%6d%8d  ", gotAny, da->acl.perm);
            aclDump (dump, &da->acl, &gotAny, 0);
        }
    }
    if (!gotAny)
        fprintf (dump,_("        --none--\n"));
}


void accessDump(FILE *dump)
{
    dll_t *save, *sp;
    dlr_t *dr;
    dla_t *da;
    int dummy = 0, allLists = 0, allEntries, gotAny;

    if (dump == NULL)
        return;

    if ((save = initChain(NULL)) == NULL) {
        errNoDynmem;
        return;
    }

/*
	scan the restriction list and the accesslist chain
	to obtain all allocated list numbers
*/

    if (restrictListH != NULL) {
        for (dr = NULL; (dr = getNextRe(restrictListH, dr)) != NULL;) {

            for (sp = NULL; (sp = getNext(save, sp)) != NULL;)
                if (sp->mark == dr->mark) break;

            if (sp == NULL) {
                if (addPlain(save, dr->mark, &dummy, sizeof(dummy))) {
                    errNoDynmem;
                    return;
                }
                allLists++;
            }
        }
    }

    if (accessListH != NULL) {
        for (da = NULL; (da = getNextACL(accessListH, da)) != NULL;) {

            for (sp = NULL; (sp = getNext(save, sp)) != NULL;)
                if (sp->mark == da->mark) break;

            if (sp == NULL) {
                if (addPlain(save, da->mark, &dummy, sizeof(dummy))) {
                    errNoDynmem;
                    return;
                }
                allLists++;
            }
        }
    }

    fprintf (dump, _("\nTotal number of ACLs allocated: %d\n"), 
             allLists);

    if (!allLists)
        return;

/*
	Now that we know all the lists, sort them by mark and dump them
*/
    sortByMark(save, DLL_SORTASC);

    for (sp = NULL; (sp = getNext(save, sp)) != NULL;)  {

        fprintf (dump, "\nACL %d\n",sp->mark);

        for (da = NULL, gotAny = 0, allEntries = 0; 
             (da = getNextACL(accessListH, da)) != NULL;)
            if (sp->mark == da->mark) {
                char *permbuf;
                allEntries++;
                permbuf = perm2names(da->acl.perm);
                fprintf (dump, "    %-17s ", permbuf);
                if (permbuf != NULL)
                    Free(permbuf);
                aclDump (dump, &da->acl, &gotAny, 1);
            }

        if (allEntries) {
            if (!gotAny)
                fprintf (dump, "    %-17s any\n", "deny");
        }
        else                       
            fprintf (dump, "    %-17s any\n", "read,write");

        dr = getNextRe(restrictListH, NULL);

        if (dr != NULL) {
            int gotAnyR = 0;
            fprintf (dump, _("\n    File restrictions for ACL %d\n"),
                     sp->mark);
            do {
                if (sp->mark == dr->mark) {
                    fprintf(dump, "        %-30s  %10ld  %d\n",
                            dr->raw, dr->maxsize, dr->filter);
                    gotAnyR++;
                }
            } while ((dr = getNextRe(restrictListH, dr)) != NULL);
            if (!gotAnyR)
                fprintf (dump,_("        --none--\n"));
        }

        dr = getNextRe(mapListH, NULL);

        if (dr != NULL) {
            int gotAnyR = 0;
            fprintf (dump, _("\n    File name maps for ACL %d\n"),
                     sp->mark);
            do {
                if (sp->mark == dr->mark) {
                    fprintf(dump, "        %-30s  %s%-30s\n",
                            dr->raw, dtype2prefix(dr->dtype), dr->map);
                    gotAnyR++;
                }
            } while ((dr = getNextRe(mapListH, dr)) != NULL);
            if (!gotAnyR)
                fprintf (dump,_("        --none--\n"));
        }
    }
}

char *dumpbList (dlb_t *bl)
{
    dlb_t *p;
    char *buf=NULL, *nbuf=NULL, cval[64];
    int size=1;

    for (p = NULL; (p = getNextBlist(bl, p)) != NULL;) {
        if (buf != NULL)
            strcat(buf, ",");
        if (p->last == INT_MAX)
            sprintf(cval, "%d-", p->first);
        else if (p->first == p->last)
            sprintf(cval, "%d", p->first);
        else
            sprintf(cval, "%d-%d", p->first, p->last);
        size += strlen(cval)+1;
        nbuf = Realloc(buf, size);
        if (nbuf == NULL)
            break;
        if (buf == NULL)
            *nbuf = '\0';
        buf = nbuf;
        strcat(buf, cval);
    }
    if (nbuf != NULL)
        return buf;
    
    if (buf != NULL)
        free(buf);
    return NULL;
}

#ifdef FILE_CACHE

void cacheDump(FILE *dump, int what)
{
    dlr_t *dr;
    dlc_t *dc;
    int first;
    char *prop;
    static char datbuf[16];
    struct tm *ltime;


    if (dump == NULL)
        return;

    fprintf (dump, _("\nFile Cache\n"));

    if (what & 1) {
        fprintf (dump, _("    Global cache limit: %10lu bytes\n"
                         "    Cache object limit: %10lu bytes\n"), 
                         maxFilecache, maxCacheobj);
        if (maxFilecache)
            fprintf (dump, 
                     _("    Current cache size: %10lu bytes (%d files)\n"),
                     currCachesize, currCachefiles);
        else
            return;
        
        fprintf (dump, _("\n    Eligible cache objects:\n"));
        for (dr = NULL, first = 1; (dr = getNextRe(objListH, dr)) != NULL;) {
            if (first) {
                first = 0;
            }
            prop = bmap2pat(dr->filter, cproplist, ",");
            fprintf (dump, "        %-30s  %s\n", dr->raw, prop);
            if (prop != NULL)
                Free(prop);
        }
        if (first)
            fprintf (dump,_("        --none--\n"));
    } 

    if (what & 2) {
        fprintf (dump, _("\n    Currently cached files:\n"));
        for (dc=NULL, first=1; (dc = getNextCache(cacheListH, dc)) != NULL;) {
            if (first) {
                first = 0;
                fprintf (dump, "        %10s %-6s %-14s %s\n", 
                         _("size"), _("flags"), _("date"), _("file name"));
            }
        
            ltime = localtime((time_t *)&dc->used);
            strftime(datbuf, sizeof(datbuf), "%Y%m%d%H%M%S", ltime);
            fprintf (dump, "        %10lu %1d %4.4x %-14s %s\n",
                     dc->size, dc->mark, dc->flags, datbuf, dc->name);

        }
        if (first)
            fprintf (dump,_("        --none--\n"));
    }
}
#endif /* FILE_CACHE */
